"""
description: this module provides exceptions of testcase.
"""


class UnsupportParameterTypeException(Exception):
    """
    description: if the parameters' type are not tuple or dict,
                 raise this exception.
    """

    def __init__(self, parameter):
        super().__init__(
            "the type of parameter '{parameter}' is '{type}', "
            "but the 'tuple' or 'dict' is expected".format(parameter=parameter, type=parameter.__class__.__name__)
        )


class UnsupportStartTypeException(Exception):
    """
    description: if the format of start are not \'xxxx.end\' or \'boolean
                 expression consisting of xxxx.end\', raise this exception.
                 (xxxx is function name of teststep)
    """

    def __init__(self, parameter):
        super().__init__(
            "the start is '{parameter}', "
            "but the format of 'xxxx.end' or 'boolean expression consisting "
            "of xxxx.end' is expected.(xxxx is function name of teststep)".format(parameter=parameter)
        )


class TestCaseException(Exception):
    """
    description: this is the base exception.
    """


class DuplicatedTestStepNameException(TestCaseException):
    """
    description: if a testcase has duplicated step names, raise this exception.
    """

    def __init__(self, teststep_name):
        super().__init__("duplicated teststep name '{teststep_name}'".format(teststep_name=teststep_name))


class HasNoAttributeException(TestCaseException):
    """
    description: if access a nonexisting attribute, raise this exception.
    """

    def __init__(self, class_name, attribute_name, file_path, line_number):
        message = f"in '{file_path}:{line_number}', class '{class_name}' has not attribute '{attribute_name}'"
        super().__init__(message)


class TestCaseStatusException(Exception):
    """
    description: the base exception of the testcase status exceptions.
    """


class PassedException(TestCaseStatusException):
    """
    description: when user sets the testcase status to passed,
                 raise this exception.
    """

    def __init__(self, line_number):
        super().__init__(f"this testcase has been set to passed, in line {line_number}")


class FailedException(TestCaseStatusException):
    """
    description: when user sets the testcase status to failed,
                 raise this exception.
    """

    def __init__(self, line_number):
        super().__init__(f"this testcase has been set to failed, in line {line_number}")


class InvestigatedException(TestCaseStatusException):
    """
    description: when user sets the testcase status to investigated,
                 raise this exception.
    """

    def __init__(self, line_number):
        super().__init__(f"this testcase has been set to investigated, in line {line_number}")


class UnavailableException(TestCaseStatusException):
    """
    description: when user sets the testcase status to unavaiable,
                 raise this exception.
    """

    def __init__(self, line_number):
        super().__init__(f"this testcase has been set to unavailable, in line {line_number}")


class BlockedException(TestCaseStatusException):
    """
    description: when user sets the testcase status to blocked,
                 raise this exception.
    """

    def __init__(self, line_number):
        super().__init__(f"this testcase has been set to blocked, in line {line_number}")


class UnexecutedException(TestCaseStatusException):
    """
    description: when user sets the testcase status to unexecuted,
                 raise this exception.
    """

    def __init__(self, line_number):
        super().__init__(f"this testcase has been set to Unexecuted, in line {line_number}")
